/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import LuminicBox.Log.Logger;
import LuminicBox.Log.ConsolePublisher;

import org.flowplayer.playlist.Clip;
import org.flowplayer.*;
import org.flowlib.*;
import org.flowplayer.playlist.PlayList;

class org.flowplayer.AbstractDurationTrackingMediaControl extends Observable {
	private var logger:Logger;

	private var playing:Boolean;
	private var paused:Boolean;
	private var timeUpdateId:Number;
	private var startTime:Number;
	private var time:Number;
	private var selfPause : Boolean;
	private var clip : Clip;
	private var context: MediaContext;

	function AbstractDurationTrackingMediaControl() {
		this.logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		resetState();
	}
		
	private function resetState():Void {
		this.time = 0;
		this.playing = false;
		this.paused = false;
		this.selfPause = false;
	}
	
	public function reset():Void {
		stop();
	}

	public function toggleLoop():Void {
	}

	public function isLooping():Boolean {
		return false;
	}

	public function startBuffering(clip:Clip):Void {
		this.clip = clip;
		this.time = 0;
		onStartBuffering(clip);
	}
	
	private function onStartBuffering(clip:Clip):Void {
		// possibly overridden in a subclass
	}
	
	
	/**
	 * @see MediaControl.playCurrent()
	 */
	public function playCurrent() : Void {
		throw new Error("Unsupported method!");
	}

	public function play(clip:Clip):Void {
		logger.debug("AbstractDurationTrackingMediaControl.play");
		this.clip = clip;
		this.playing = true;
		this.paused = false;

		// we just load it every time since after the first load it's cached
		startBuffering(clip);
			
		selfPause = pauseIfDurationelessClip();
		if (selfPause) return;

		var stopped:Boolean = stopIfDurationlessClip();
		if (stopped) {
			this.selfPause = false;
			return;
		}
			
		createTimeUpdateInterval();
	}
	
	private function createTimeUpdateInterval():Void {		
		this.startTime = getTimer();
		updatePlaybackTime.controller = this;
		logger.debug("Creating time update interval");
		this.timeUpdateId = setInterval(updatePlaybackTime, 200);
	}
	
	private function pauseIfDurationelessClip():Boolean {
		if (selfPause) {
			logger.debug(this + " selfPaused");
			return false;
		}
		if (clip.getDuration() == 0) {
			// no time specified for this clip --> pause and return
			this.time = 0;
			pause();
			dispatchEvent("onSelfPause");
			return true;
		}
		return false;
	}
	
	private function stopIfDurationlessClip():Boolean {
		if (! clip > 0) {
			stop();
			selfPause = false;
			dispatchClipDone();
			return true;
		}
		return false;
	}

	private function deleteTimeUpdateInterval():Void {
		clearInterval(timeUpdateId);
	}
	
	private function updatePlaybackTime():Void {
		var controller = arguments.callee.controller;
		controller.doUpdatePlaybackTime();
	}
	
	public function doUpdatePlaybackTime():Void {
		logger.debug("AbstractDurationTrackingMediaControl: doUpdatePlaybackTime, paused " + paused);
		if (paused) return;
		
		var timeNow:Number = getTimer();
		this.time += (timeNow - startTime)/1000;
		logger.debug("playback time " + time);
		logger.debug("clip: " + clip);
		if (time >= clip.getDuration()) {
			logger.debug("AbstractDurationTrackingMediaControl: doUpdatePlaybackTime, clip done, stopping.");
			stop();
			dispatchClipDone();
		}
		
		onTimeTrackingInterval();
	}
	
	private function dispatchClipDone():Void {
		dispatchEvent("onClipDone");
	}
	
	private function onTimeTrackingInterval():Void {
		// possibly overridden in a subclass
	}

	public function stop():Void {
		logger.debug(this + ".stop");
		resetState();
		deleteTimeUpdateInterval();
	}

	public function pause():Void {
		deleteTimeUpdateInterval();
		this.paused = true;
	}

	public function resume():Void {
		this.paused = false;
		logger.debug("AbstractDurationTrackingMediaControl.resume");
		if (selfPause)
			play();
		else
			createTimeUpdateInterval();
	}

	public function seek(percentage:Number):Void {
		this.time = (percentage / 100) * clip.getDuration();  
	}

	public function seekToTime(offset:Number):Void {
		this.time = offset;
	}

	public function isPaused():Boolean {
		return this.paused;
	}

	public function isPlaying():Boolean {
		return this.playing;
	}

	public function getPlaybackStatus():PlaybackStatus {
		return new PlaybackStatus(clip.getDuration() || 0, time, 0, 0);
	}

	/*
	 * Notification that the current clip has changed in the playlist.
	 */
	public function onClipChanged():Void {
	}

	/*
	 * @see MediaControl#attachVideo()
	 */
	public function attachVideo(video:Video, mediaType:String):Void {
		// this method has no effect in this class
	}

	public function setContext(context:MediaContext) {
		this.context = context;
	}

	public function stopCurrent():Void {
		stop();
	}

	public function addCuePoint(cueTimeSeconds : Number, cueName : String, cueObject : Object) : Void {
	}

	public function addCuePoints(cuePoints:Array):Void {
	}

	public function setPlayList(playList:PlayList):Void {
	}

}
