/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowplayer.MediaContext;
import org.flowplayer.PlaybackStatus;
import org.flowplayer.MediaSize;
import org.flowplayer.playlist.Clip;
import org.flowplayer.MediaView;
import org.flowplayer.playlist.PlayList;

/**
 * VideoControl is an interface used to control the playback of a video stream.
 * It provides methods for starting, stopping, pausing and resuming the playback.
 * <p>
 * It also provides the {@link #addListener()} method that is used to add a listener
 * for different events occurring in while the video is playing.
 */
interface org.flowplayer.MediaControl {
		
	public function setPlayList(playList:PlayList):Void;
	
	/**
	 * Attach a video display to this controller.
	 */
	public function attachView(mcDisplay:MovieClip, mediaType:String):Void;

	public function attachVideo(video:Video, mediaType:String):Void;

	public function setContext(context:MediaContext);
	
	public function setSound(sound:Sound):Void ;
	/**
	 * Toggles looping.
	 */ 
	public function toggleLoop():Void;

	/**
	 * Is this plauer looping?
	 */ 
	public function isLooping():Boolean;

	/**
	 * Starts loading the video stream into buffer.
	 */ 
	public function startBuffering(clip:Clip):Void;

	/**
	 * Plays according to the playlist supplied to this control.
	 * Resumes playback if paused.
	 */ 
	public function playCurrent():Void;

	/**
	 * Plays the specified clip.
	 * Resumes playback if paused.
	 */ 
	public function play(clip:Clip):Void;
	
	/**
	 * Stops the playback and goes to clip 1.
	 */ 
	public function stop():Void;
	
	/**
	 * Rewinds to the start. Same as stop if not playing.
	 * If playing, seek to beginning, pause and leave buffering.
	 */
	public function rewind():Void;
	
	/**
	 * Stops playing the current clip.
	 */ 
	public function stopCurrent():Void;
	
	/**
	 * Pauses playback.
	 */ 
	public function pause():Void;
	
	/**
	 * Resumes paused playback.
	 */ 
	public function resume():Void;
	
	/**
	 * Moves the playback position to the position designated
	 * by the specified percentage value.
	 *
	 * @param percentage the position where to seek as percentage 
	 *                   of the total video length
	 */ 
	public function seek(percentage:Number):Void;
	
	/**
	 * Moves the playback position to the position designated
	 * by the specified time value.
	 *
	 * @param time offset from the beginning of the video, milliseconds
	 */ 
	public function seekToTime(offset:Number):Void;
	
	/**
	 * Pauses the video if it is currently playing or resumes
	 * it if it is paused.
	 */ 
	//public function pauseOrResume():Void;
	
	/**
	 * Is this controller paused?
	 */ 
	public function isPaused():Boolean;
	
	/**
	 * Is this controller currently playing the video?
	 */ 
	public function isPlaying():Boolean;
	
	/**
	 * Gets the playback status.
	 *
	 * @return percentage played from the total lenth of the video
	 */ 
	public function getPlaybackStatus():PlaybackStatus;
	
	/**
	 * Adds an event listener.
	 *
	 * @param eventName name of the event to listen
	 * @param targetObject listener object to be notified about the specified event
	 */ 
	public function addListener(eventName:String, targetObject:Object);
	
	/**
	 * Notifies this controller that the video clip has been changed. The controller
	 * should synchronize with it's underlying clip source (i.e. a playlist):
	 */
	public function onClipChanged():Void;
	
	/**
	 * Reset this controller to it's initial state.
	 */
	public function reset():Void;
	
	/**
	 * Adds a programmatic cue point. 
	 */
	public function addCuePoint(cueTimeSeconds:Number, cueName:String, cueParams:Object):Void;

	public function addCuePoints(cuePoints:Array):Void;

	/**
	 * Does this controller set it's own view's visibility by itself?
	 */
	public function setViewVisibility():Boolean;

	/**
	 * Does this controller allow it's view to be centered on loadBegin?
	 */
	public function allowCenterViewOnLoadBegin():Boolean;

}