﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.Observable;
import org.flowplayer.playlist.Clip;
import LuminicBox.Log.Logger;
import LuminicBox.Log.ConsolePublisher;

/**
 * @author anssi
 */
class org.flowplayer.playlist.PlayList extends Observable {
	private var logger:Logger = new Logger();
	private var clips:Array;
	private var current:Number;
	public static var CLIP_CHANGED_EVENT:String = "onClipChanged";
	private static var OVERLAY_PLAY_CLIP:Clip = Clip.createFromObject(undefined, { overlayId: 'play' }); 
	
	private function PlayList(clips:Array) {
		this.clips = clips;
		current = 0;
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		logger.info("Instantiated playList with clips:");
		logger.info(this.clips);
	}
	
	public static function forOneClip(clipName:String, baseUrl:String, clipFileName:String, usePlayOverlay:Boolean, splashImageURL:String):PlayList {
		if (splashImageURL != undefined)
			return new PlayList([Clip.createSimple(baseUrl, splashImageURL), Clip.createSimple(baseUrl, clipFileName, clipName)]);
		if (usePlayOverlay)
			return new PlayList([OVERLAY_PLAY_CLIP, Clip.createSimple(baseUrl, clipFileName, clipName)]);
		else
			return new PlayList([Clip.createSimple(baseUrl, clipFileName, clipName)]);
	}
	
	public static function forURL(url:String):PlayList {
		return new PlayList([Clip.createSimple(undefined, url, undefined)]);
	}
	
	public static function fromObjectsWithProps(baseURL:String, objects:Array, usePlayOverlay:Boolean, splashImageURL:String, hasStreamingServer:Boolean):PlayList {
		return new PlayList(createClips(baseURL, objects, usePlayOverlay, splashImageURL, hasStreamingServer, true));
	}
	
	public function appendObjectsWithProps(baseURL:String, objects:Array, usePlayOverlay:Boolean, splashImageURL:String, hasStreamingServer:Boolean, addToTop:Boolean):Void {
		if (addToTop)
			clips = createClips(baseURL, objects, usePlayOverlay, splashImageURL, hasStreamingServer, true).concat(clips);
		else
			clips.concat(createClips(baseURL, objects, usePlayOverlay, splashImageURL, hasStreamingServer, true));
	}
	
	public static function createClips(baseURL:String, objects:Array, usePlayOverlay:Boolean, splashImageURL:String, useParentChildRelationshipForAdjacentFLVs:Boolean, expandSuggestionClips:Boolean):Array {
		var clips:Array = new Array();
		
		if (splashImageURL)
			clips.push(Clip.createSimple(baseURL, splashImageURL));
		if (usePlayOverlay && objects[0].overlayId == undefined)
			clips.push(OVERLAY_PLAY_CLIP);
		
		var clip:Clip, parentClip:Clip;
		for (var i = 0; i < objects.length; i++) {
			var clipObj:Object = objects[i];
			clip = Clip.createFromObject(baseURL, clipObj);
			if (useParentChildRelationshipForAdjacentFLVs && clip.getType() == 'flv' && parentClip.getType() == 'flv') {
				parentClip.addChild(clip);
			} else {
				clips.push(clip);
				parentClip = clip;
			}
			if (clipObj.suggestedClipsInfoUrl && expandSuggestionClips)
				clips.push(Clip.createSuggestionInfo(baseURL, clipObj.suggestedClipsInfoUrl));
		}
		return clips;
	}

	public function all():Array {
		var all:Array = new Array();
		for (var i : Number = 0; i < clips.length; i++) {
			var clip:Clip = clips[i];
			all.push(clip);
			all = all.concat(clip.getChildren());
		}
		return all;
	}

	public function allParents():Array {
		return (new Array()).concat(clips);
	}
	
	public function hasNext():Boolean {
		return current < clips.length - 1; 
	}
	
	public function next():Clip {
		logger.debug("PlayList.next(), current index = " + current);
		if (current == clips.length -1) return null;
		if (checkPlayCountAndRemoveIfReached(++current)) {
			next();
		}
		var clip:Clip = clips[current];
		dispatchEvent(PlayList.CLIP_CHANGED_EVENT, clip);
		return clips[current];
	}

	private function checkPlayCountAndRemoveIfReached(index:Number):Boolean {
		var removed:Boolean = false;
		var clip:Clip = clips[index];
		if (clip.maxPlayCountReached()) {
			removeClipAt(index);
			removed = true;
		}
		return removed;
	}

	public function peekNext():Clip {
		if (current == clips.length -1) return null;
		if (checkPlayCountAndRemoveIfReached(current + 1)) {
			return peekNext();
		}
		return clips[current + 1];
	}
	
	public function peekPrevious():Clip {
		if (current == 0) return null;
		if (checkPlayCountAndRemoveIfReached(current - 1)) {
			return peekPrevious();
		}
		return clips[current - 1];
	}
	
	public function prev():Clip {
		logger.debug("PlayList.prev(), current index = " + current);
		if (current == 0) return null;
		if (checkPlayCountAndRemoveIfReached(--current)) {
			prev();
		}
		var clip:Clip = clips[current];
		dispatchEvent(PlayList.CLIP_CHANGED_EVENT, clip);
		return clips[current];
	}
	
	public function currentClip():Clip {
		return clips[current];
	}
	
	public function currentNonOverlay():Clip {
		var current:Clip = currentClip();
		if (current.getType() == 'overlay') {
			return currentIndex() == 0 ? peekNext() : peekPrevious();
		} else
			return current;
	}
	
	public function replaceCurrentWith(clip:Clip):Void {
		clips[current] = clip;
	}
	
	public function replace(clipToReplace:Clip, replaceWith:Clip):Void {
		clips[indexOf(clipToReplace)] = replaceWith;
	}
	
	public function size():Number {
		var size:Number = 0;
		for (var i : Number = 0; i < clips.length; i++) {
			var clip:Clip = clips[i];
			size += (1 + clip.getChildren().length);
		}
		return size;
	}
	
	public function currentIndex():Number {
		return current;
	}
	
	public function toIndex(index:Number, silent:Boolean):Clip {
		if (index < 0) return null;
		if (index >= clips.length) return null;
		var clip:Clip = clips[index];
		if (index == current) return clip;
		current = index;
		if (!silent)
			dispatchEvent(PlayList.CLIP_CHANGED_EVENT, clip);
		return clip;
	}

	public function toClip(clip:Clip, silent:Boolean):Void {
		toIndex(indexOf(clip), silent);
	}
	
	private function indexOf(clip:Clip):Number {
		for (var i : Number = 0; i < clips.length; i++) {
			if (clips[i] == clip) return i;
		}
		return -1;
	}
	
	public function toString():String {
		return "PlayList: " + clips;
	}
	
	public function getClipTypes():Array {
		var types:Object = new Object();
		for (var i : Number = 0; i < clips.length; i++) {
			var clip:Clip = clips[i];
			types[clip.getType()] = 1;
		}
		var typeArray:Array = new Array();
		for (var type in types) {
			typeArray.push(type);
		}
		return typeArray;
	}
			
	public function firstClipIsSplash():Boolean {
		var firstClip:Clip = clips[0];
		return firstClip.isImage() && firstClip.getDuration() == 0;
	}

	public function addClipAfterFirstVideo(clip:Clip):Number {
		var insertionPoint:Number = findFirstFLVFromCurrent() + 1 || clips.length;
		addClip(clip, insertionPoint);		 
		return insertionPoint;
	}
	
	public function addClipAfterAnother(clip:Clip, another:Clip):Void {
		addClip(clip, indexOf(another)+1);
	}
	
	private function addClip(clip:Clip, insertionPoint:Number):Void {
		logger.debug("adding clip " + clip + ", insertionPoint " + insertionPoint);
		var newClips:Array = clips.slice(0, insertionPoint);
		newClips.push(clip);
		for (var i:Number = insertionPoint; i < clips.length; i++) {
			newClips.push(clips[i]);
		}
		logger.debug("new playlist after clip insertion:");
		logger.debug(newClips);
		this.clips = newClips;
	}

	private function findFirstFLVFromCurrent() : Number {
		for (var i:Number = current; i < clips.length; i++) {
			if (clips[i].getType() == 'flv')
				return i;
		}
		return undefined;
	}

	private function removeClipAt(index:Number):Void {
		var newClips:Array = clips.splice(0, index);
		for (var i:Number = 1; i < clips.length; i++) {
			newClips.push(clips[i]);
		}
		logger.debug("new playlist after clip removal:");
		logger.debug(newClips);
		this.clips = newClips;
	}

	public function remove(clip:Clip):Void {
		removeClipAt(current);
	}

}