import asunit.framework.TestCase;
import org.flowplayer.config.*;
import org.flowplayer.playlist.*;
import org.flowplayer.skin.Skin;
import LuminicBox.Log.*;

class org.flowplayer.config.HierarchicalConfigTest extends TestCase {

	private var logger:Logger;
	
	public function HierarchicalConfigTest(name:String) {
		super(name);
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
	}
	
	/*
	 ************ autoPlay
	 */
	 
	public function testIsAutoPlayTopmostOverrides():Void {
		var config:ExtendedConfig = new HierarchicalConfig("{ autoPlay: false }", "topmost config");
		var config2:ExtendedConfig = new HierarchicalConfig("{ autoPlay: true }", "child config");
		config.addChild(config2);
		
		assertFalse("did not used topmost config", config.isAutoPlay());
	}
	
	public function testIsAutoPlayIsTrueByDefault():Void {
		var config:ExtendedConfig = new HierarchicalConfig(undefined, "empty config");
		assertTrue("did not return true by default", config.isAutoPlay());
	}
	
	public function testIsAutoPlayTrue():Void {
		var config:ExtendedConfig = new HierarchicalConfig("{ autoPlay: true }", "config with autoPlay false");
		assertTrue("autoPlay did not go to true when overriding default", config.isAutoPlay());
	}
	
	public function testIsAutoPlayTrueWhenUsingSplash():Void {
		var config:ExtendedConfig = new HierarchicalConfig("{ autoPlay: false, splashImageFile: 'foobar.jpg', videoFile: 'video' }", "config with autoPlay false");
		assertTrue("autoPlay was not true with splash", config.isAutoPlay());		
	}

	public function testIsAutoPlayFalseWhenSplashHasDuration():Void {
		var configString2:String = "{ autoPlay: false, playList: [{ url: 'main_clickToPlay.jpg', duration: 1 }, { name: 'Honda Accord', url: 'honda_accord.flv' }] }";
		var config2:ExtendedConfig = new HierarchicalConfig(configString2, "config with playlist having splash+duration");
		assertFalse("autoPlay was true when the splash had a duration", config2.isAutoPlay());		
	}	

	public function testIsAutoPlayFalseWithFLVPlayList():Void {
		var configString3:String = "{ autoPlay: false, playList: [{ url: 'main_clickToPlay.flv' }, { name: 'Honda Accord', url: 'honda_accord.flv' }] }";
		var config3:ExtendedConfig = new HierarchicalConfig(configString3, "config with playlist but no splash");
		assertFalse("autoPlay was true with a FLV playlist", config3.isAutoPlay());		
	}	
	
	public function testIsAutoPlayTrueWhenSplashInPlayList():Void {
		var configString:String = "{ autoPlay: false, playList: [{ url: 'main_clickToPlay.jpg' }, { name: 'Honda Accord', url: 'honda_accord.flv' }] }";
		var config:ExtendedConfig = new HierarchicalConfig(configString, "config with playlist having splash");
		assertTrue("autoPlay was not true with splash in playlist", config.isAutoPlay());		
	}	
	
	/*
	 ************ autoBuffering
	 */
	
	public function testIsAutoBufferingIsTrueByDefault():Void {
		var config:ExtendedConfig = new HierarchicalConfig(undefined, "empty config");
		assertTrue("did not return autoBuffering true by default", config.isAutoBuffering());
	}
	 
	public function testIsAutoBufferingFalse():Void {
		var config:ExtendedConfig = new HierarchicalConfig("{ autoBuffering: false }", "config with autoBuffering false");
		assertFalse("autoBuffering did not go false when overriding default", config.isAutoBuffering());
	}

	/*
	 ************ playList
	 */
	
	public function testCachesPlayList():Void {
		var configString:String = "{ autoPlay: false, playList: [{ url: 'main_clickToPlay.jpg' }, { name: 'Honda Accord', url: 'honda_accord.flv' }] }";
		var config:ExtendedConfig = new HierarchicalConfig(configString, "config with playlist having splash");
		
		var pl1:PlayList = config.getPlayList();
		var pl2:PlayList = config.getPlayList();
		assertEquals(pl1, pl2);
	}
	
	public function testGetPlaylist():Void {
		var baseUrl:String = "http://www.videos.org";
		var file1:String = "main_clickToPlay.jpg";
		var file2:String = "honda_accord.flv";
		var config:ExtendedConfig = new HierarchicalConfig("{ baseURL: '"+baseUrl+"', playList: [{ url: '"+file1+"' }, { name: 'Honda Accord', url: '"+file2+"' }] }", "config with a playlist");
		var playList:PlayList = config.getPlayList();
		var clip1:Clip = playList.all()[0];
		var clip2:Clip = playList.all()[1];
		logger.debug(clip1);
		assertEquals(baseUrl + "/" + file1 +  clip1.getURL());
		assertEquals(baseUrl + "/" + file2 +  clip2.getURL());
	}
	
	public function testSplashURLWhenSpecifiedOutsidePlaylist():Void {
		var baseUrl:String = "http://www.videos.org";
		var file1:String = "main_clickToPlay.jpg";
		var file2:String = "honda_accord.flv";
		var config:ExtendedConfig = new HierarchicalConfig("{ baseURL: '"+baseUrl+"', playList: [ { name: 'Honda Accord', url: '"+file2+"' }], splashImageFile: '"+file1+"' }", "config with a playlist");		
		var splash:Clip = config.getPlayList().all()[0];
		assertEquals("splash url was: " + splash.getURL(), baseUrl + "/" + file1, splash.getURL());
	}
	
	/*
	 ************ baseURL
	 */
	
	public function testGetBaseURL():Void {
		var urlValue:String = "urlie";
		var config:ExtendedConfig = new HierarchicalConfig("{ baseURL: '"+urlValue+"' }");
		assertEquals("baseURL did not match", urlValue, config.getBaseURL());
	}
	
	public function testGetBaseURLHasNoDefault():Void {
		assertEquals(undefined, emptyConfig().getBaseURL());
	}
	
	public function testGetBaseURLIsEmptyWhenStreamingServerURLDefined():Void {
		var config:ExtendedConfig = new HierarchicalConfig("{ streamingServerURL: 'rtmp://localahost:oflaDemo' }");
		var url:String = config.getBaseURL();
		assertEquals("baseURL was '" + url + "'", '', url);
	}
	
	/*
	 ************ loop
	 */
	
	public function testIsLoopTrueByDefault():Void {
		assertTrue("was not true by default", emptyConfig().isLooping());
	}
	
	public function testIsLoop():Void {
		assertTrue("did not return true", (new HierarchicalConfig("{ loop: true }")).isLooping());
		assertFalse("did not return false", (new HierarchicalConfig("{ loop: false }")).isLooping());
	}

	/*
	 ************ buffer lenfths
	 */
	
	public function testStrtingBufferLength():Void {
		assertEquals(18, (new HierarchicalConfig("{ startingBufferLength: 18 }")).getStartingBufferLength());
	}
	
	public function testStartingBufferLengthEqualsToMainBufferLengthIfNotDefined():Void {
		var config:HierarchicalConfig = new HierarchicalConfig("{ bufferLength: 16 }");
		var mainBufferLength:Number = config.getMainBufferLength();
		assertEquals("wrong main buffer length", 16, mainBufferLength);
		assertEquals("starting buffer lenth was not equal to main buffer length", mainBufferLength, config.getStartingBufferLength());
	}
	
	public function testEqualBufferLengthDefaults():Void {
		var config:ExtendedConfig = emptyConfig();
		var value:Number = config.getMainBufferLength();
		assertTrue("default main buffer length not bigger than zero", value > 0);
		assertEquals("not equal", value, config.getStartingBufferLength());
	}

	public function testMainBufferLength():Void {
		assertEquals(18, (new HierarchicalConfig("{ bufferLength: 18 }")).getMainBufferLength());
	}
	
	/*
	 ************ videoHeight
	 */
	
	public function testGetVideoHeightDoesNotHaveDefault():Void {
		assertEquals(undefined, emptyConfig().getVideoHeight());
	}
	
	public function testGetVideoHeight():Void {
		assertEquals(182, (new HierarchicalConfig("{ videoHeight: 182 }")).getVideoHeight());		
	}
	
	public function testGetVideoHeightIsUndefinedInFullScreen():Void {
		assertEquals(undefined, (new HierarchicalConfig("{ fullScreen: true, videoHeight: 182 }")).getVideoHeight());		
	}

	/*
	 ************ showControls
	 */
	
	public function testShowControlsTrueByDefault():Void {
		assertTrue(emptyConfig().showControls());
	}
	
	public function testShowControls():Void {
		var config:ExtendedConfig = emptyConfig();
		assertTrue("was not true", (new HierarchicalConfig("{ hideControls: false }")).showControls());		
		assertFalse("was not false", (new HierarchicalConfig("{ hideControls: true }")).showControls());		
	}

	/*
	 ************ showPlaylist
	 */

	public function testShowPlayListButtons():Void {
		var config:ExtendedConfig = emptyConfig();
		assertFalse("default was not false", config.showPlayListButtons());		
		assertTrue("was not true", (new HierarchicalConfig("{ showPlayListButtons: true }")).showPlayListButtons());
		
		config.addChild(new HierarchicalConfig("{ showPlayListButtons: true }", "child config"));		
		assertTrue("was not true from child config", config.showPlayListButtons());
	}

	/*
	 ************ useEmbeddedButtonImages
	 */
	
	public function testUseEmbeddedButtonImagesTrueByDefault():Void {
		assertTrue(emptyConfig().useEmbeddedButtonImages());
	}
	
	public function testUseEmbeddedButtonImages():Void {
		var config:ExtendedConfig = emptyConfig();
		assertTrue("was not true", (new HierarchicalConfig("{ useEmbeddedButtonImages: true }")).useEmbeddedButtonImages());
		assertFalse("was not false", (new HierarchicalConfig("{ useEmbeddedButtonImages: false }")).useEmbeddedButtonImages());		
	}

	/*
	 ************ skin
	 */
	 
	 public function testGetSkinImagesBaseURL():Void {
	 	var url:String = "http://www.flowplayerskinmarket.com";
	 	assertEquals(url, (new HierarchicalConfig("{ skinImagesBaseURL: '"+url+"' }")).getSkinImagesBaseURL());
	 }
	 
	 public function testGetLogoFile():Void {
	 	var logo:String = "logo.jpg";
	 	assertEquals(logo, (new HierarchicalConfig("{ logoFile: '"+logo+"' }")).getLogoFile());
	 }
	
	private function emptyConfig():ExtendedConfig {
		return new HierarchicalConfig("{}", "empty config");
	}
	
	/*
	 ************ serialization
	 */
	
	public function testStringify():Void {
		var stringConfig:String = "{ playList: [ { url: 'url1', name: 'name1' }, { url: 'url2', name: 'name2' } ], autoPlay: true, autoBuffering: true }";
		var config:Config = new HierarchicalConfig(stringConfig, "test config");
		var newConfig:Config = new HierarchicalConfig(config.stringify(), "new config");
		assertTrue("autobuffering", config.isAutoBuffering());
		assertTrue("autoPlay", config.isAutoPlay());
		var playList:PlayList = newConfig.getPlayList();
		assertEquals("url1", 'url1', Clip(playList.all()[0]).getURL());
		assertEquals("name1", 'name1', Clip(playList.all()[0]).getName());
		assertEquals("url2", 'url2', Clip(playList.all()[1]).getURL());
		assertEquals("name2", 'name2', Clip(playList.all()[1]).getName());
	}
	
	public function testStringifyWithChildConfig():Void {
		var config:ExtendedConfig = new HierarchicalConfig("{ baseURL: 'test', configFileName: 'foobar.js', showMenu: false }");
		assertEquals('foobar.js', config.getConfigFileName());

		var child:ExtendedConfig = new HierarchicalConfig("{ videoFile: 'foobar.flv', showMenu: true }");
		config.addChild(child);
		var stringConf:String = config.stringify();
		logger.error("Stringified config: " + stringConf);
		var newConfig:ExtendedConfig = new HierarchicalConfig(stringConf, "new config");
		logger.error(newConfig.getConfigObj());
		logger.error(newConfig.getChild());
		assertEquals("config file name", 'foobar.js', newConfig.getConfigFileName());
		logger.error("video file: " + newConfig.getVideoFile());
		assertEquals("video file name", "test/foobar.flv", newConfig.getVideoFile());
		assertFalse("show menu", newConfig.showMenu());
	}
	
	/*
	 ************ menu items
	 */
	
	public function testShowMenuItemShowsAllByDefault():Void {
		var config:Config = emptyConfig();
		assertTrue(config.showMenuItem(0));
		assertTrue(config.showMenuItem(1));
		assertTrue(config.showMenuItem(2));
		assertTrue(config.showMenuItem(3));
		assertTrue(config.showMenuItem(4));
		assertTrue(config.showMenuItem(5));
	}
	
	public function testShowMenuItem():Void {
		var config:Config = new HierarchicalConfig("{ menuItems: [ false, true, false, false, true, false ] }");
		assertFalse(config.showMenuItem(0));
		assertTrue(config.showMenuItem(1));
		assertFalse(config.showMenuItem(2));
		assertFalse(config.showMenuItem(3));
		assertTrue(config.showMenuItem(4));
		assertFalse(config.showMenuItem(5));
	}
}
